#include <LiquidCrystal.h> // librairie lcd
LiquidCrystal lcd(12, 11, 1,4, 3, 2);// broches utilisées pour communiquer

char inputPin = 5; // entrée T1 pour Timer1 tension aux bornes de la résistance
volatile uint16_t count; //variable 16 bits qui contiendra le nombre de flashs lumineux par seconde
volatile uint16_t ni; // compteur d'interruptions
uint16_t ninter = 60; // nombre d'interruptions pour un comptage 
                       
void start_count() { 
  
// Timer 2 : génération d'interruptions périodiques (genere des impulsions toutes les 1/60 seconde)

// Timer en mode CTC (clear timer on compare), qui consiste à remettre à zéro le registre du compteur 8 bits (TCNT2)lorsqu'il
//atteint la valeur du registre OCR2A. On choisit OCR2A=0xFF, ce qui signifie que la période d'interruption est égale à 256 tops d'horloge. L'horloge
//du compteur est l'horloge de la carte (16 MHz), à laquelle une division est appliquée (prescaler). On choisit le facteur de division le plus élevé
// (1024), ce qui fait uen fréquence d'horloge de 16/1024 MHz.
 
  TCCR2A |= (1 << WGM21); //  mode CTC 
  OCR2A = 0xFF; // fréquence d'interruption 16 MHz /1024/256
  TIMSK2 = 1 << TOIE2; // mode débordement du Timer 2 (la fonction ISR(TIMER2_OVF_vect) sera appellée à chaque débordement du timer soit tout les 1/60 seconde
  TCNT2 = 0;// mise à 0 du registre
  ni = 0; // mise à 0 du compteur d'interruptions
  
  // Timer 1 : compteur d'impulsions 

// configuration en mode capture
  
  TCCR1A = 0;
  TCCR1B = 0;
  
  TCNT1 = 0;// mise à 0 du compteur 1 
  
  count = 0;// mise à 0 de la variable count
  
  sei(); // activation des interruptions
  TCCR2B |= (1 << CS12) | (1 << CS11) | (1 << CS10); // prescaler = 1024
  TCCR1B |= (1 << CS12) | (1 << CS11) | (1 << CS10); // Horloge externe sur front montant
}

// Voici la fonction appelée lors de l'interruption déclenchée par le débordement du timer 2. Lorsque le nombre d'interruptions atteint ninter, on
// enregistre le nombre d'impulsions comptées dans count puis on remet le compteur d'impulsions à zéro.

                        
ISR(TIMER2_OVF_vect) { // Fonction d'interruption déclenchée lors du débordement du timer 2
   ni++; // incrémentation de ni

// on stocke la valeur du compteur dans count lorsque ni=60 soit toutes les secondes et on remet le compteur à 0 
   
   if (ni==ninter) {
     ni = 0;
     count = TCNT1;
     TCNT1 = 0;
    
   }
}

// La fonction setup déclenche le comptage
    
void setup() {
  
 lcd.begin(16,2); // on utilise un écran LCD 16 caractères 2 lignes pour afficher la fréquence
 pinMode(inputPin,INPUT); //on définit le pin 5 comme entrant 
 
  start_count();// déclenche le comptage
  
  }

// la fonction loop affiche la valeur de la fréquence sur l'écran LCD toutes les secondes
  
  void loop() {
    
  delay(100);// on attend un peu avant de lancer une boucle
  
  float freq = count; // on stocke la valeur de count dans une variable lisible par le LCD
 
  lcd.setCursor(0,0);// mise à la ligne du curseur LCD
  lcd.print("TOURS/SECONDE");// écriture de l'unité
  lcd.setCursor(0,1); // mise à la ligne du curseur LCD
  lcd.print(freq,1);// écriture de la fréquence
 }
                          
